@@ -1,9 +1,13 @@
 import math
 import numpy
 import copy
+import time
+import pandas as pd
 from .db import CorpsEval
 from krx_hj3415 import krx
 from util_hj3415 import utils
+from stock_core.nfscrapy import scraper
+from multiprocessing import Process, cpu_count, Queue
 
 import logging
 logger = logging.getLogger(__name__)
@@ -54,9 +58,17 @@ class Eval:
 
     def __init__(self, code):
         self.code = code
-        self.corp_eval_db = CorpsEval(code)
-        self.corp_eval_db.set_col('c101')
-        self.rc101 = self.corp_eval_db.get_recent()[0]
+        try:
+            self.corp_eval_db = CorpsEval(code)
+            self.corp_eval_db.set_col('c101')
+            self.rc101 = self.corp_eval_db.get_recent()[0]
+        except StopIteration:
+            # 데이터베이스에 c101자료가 없는 경우는 직접 스크래핑을 시도한다.
+            scraper.run('c101', [code, ])
+            self.corp_eval_db = CorpsEval(code)
+            self.corp_eval_db.set_col('c101')
+            self.rc101 = self.corp_eval_db.get_recent()[0]
+        logger.debug(f'rc101:{self.rc101}')
         self.name = self.rc101['종목명']
         self.marketcap = int(self.rc101['시가총액']) / 100000000
         # self.corf_df = CorpDF(code, 'c101')
@@ -178,7 +190,10 @@ class Eval:
                 유동비율 = round(((유동자산 + 추정영업현금흐름) / 유동부채) * 100, 2)
                 logger.debug(f'{self.code} 계산된 유동비율 {유동비율}')
 
-        dict영업이익률 = self.corp_eval_db.find_c106('q', '영업이익률')
+        try:
+            dict영업이익률 = self.corp_eval_db.find_c106('q', '영업이익률')
+        except KeyError:
+            dict영업이익률 = {self.name: float('nan')}
         logger.debug(f'{self.code} 영업이익률 {dict영업이익률}')
 
         return {
@@ -733,3 +748,87 @@ class ReportOne:
             'mil_s': self.mil_scores,
             'blue_s': self.blue_scores,
         }
+
+    def for_table(self) -> dict:
+        logger.debug(f"code: {self.eval_dict['c101']['코드']}")
+        logger.debug(f"PFCF : {self.eval_dict['mil']['가치지표']['PFCF']}")
+        logger.debug(f"PCR : {self.eval_dict['mil']['가치지표']['PCR']}")
+        return {
+            'code': self.eval_dict['c101']['코드'],
+            '종목명': self.eval_dict['c101']['종목명'],
+            '주가': self.eval_dict['c101']['주가'],
+            'PER': self.eval_dict['c101']['PER'],
+            'PBR': self.eval_dict['c101']['PBR'],
+            '시가총액': self.eval_dict['c101']['시가총액'],
+            'RED': self.eval_dict['red']['red_price'],
+            '주주수익률': self.eval_dict['mil']['주주수익률'],
+            '이익지표': self.eval_dict['mil']['이익지표'],
+            'ROIC': self.eval_dict['mil']['투자수익률']['ROIC'],
+            'ROE': self.eval_dict['mil']['투자수익률']['ROE'],
+            'PFCF': utils.ext_latest_value(self.eval_dict['mil']['가치지표']['PFCF']),
+            'PCR': utils.ext_latest_value(self.eval_dict['mil']['가치지표']['PCR']),
+        }
+
+
+def _make_df_part(codes: list, q):
+    t = len(codes)
+    d = []
+    for i, code in enumerate(codes):
+        print(f'{i+1}/{t} {code}')
+        try:
+            d.append(ReportOne(code).for_table())
+        except:
+            logger.error(f'{code} / {ReportOne(code).for_table()}')
+            continue
+    df = pd.DataFrame(d)
+    logger.debug(df)
+    q.put(df)
+
+
+def make_eval_df_all() -> pd.DataFrame:
+    def _code_divider(entire_codes: list) -> tuple:
+        # 전체 종목코드를 리스트로 넣으면 cpu 코어에 맞춰 나눠준다.
+        # reference from https://stackoverflow.com/questions/19086106/how-to-utilize-all-cores-with-python-multiprocessing
+        def _split_list(alist, wanted_parts=1):
+            # 멀티프로세싱할 갯수로 리스트를 나눈다.
+            # reference from https://www.it-swarm.dev/ko/python/%EB%8D%94-%EC%9E%91%EC%9D%80-%EB%AA%A9%EB%A1%9D%EC%9C%BC%EB%A1%9C-%EB%B6%84%ED%95%A0-%EB%B0%98%EC%9C%BC%EB%A1%9C-%EB%B6%84%ED%95%A0/957910776/
+            length = len(alist)
+            return [alist[i * length // wanted_parts: (i + 1) * length // wanted_parts]
+                    for i in range(wanted_parts)]
+
+        core = cpu_count()
+        print(f'Get number of core for multiprocessing : {core}')
+        n = core - 1
+        if len(entire_codes) < n:
+            n = len(entire_codes)
+        print(f'Split total {len(entire_codes)} codes by {n} parts ...')
+        divided_list = _split_list(entire_codes, wanted_parts=n)
+        return n, divided_list
+
+    from stock_core.load_db import Settings
+    setting = Settings(echo=True)
+    setting.set_mongo(mongo_addr='mongodb://192.168.0.173:27017')
+    codes = CorpsEval.get_all_codes_in_db()
+
+    print('*' * 25, f"Eval all using multiprocess", '*' * 25)
+    print(f'Total {len(codes)} items..')
+    logger.info(codes)
+    n, divided_list = _code_divider(codes)
+
+    start_time = time.time()
+    q = Queue()
+    ths = []
+    for i in range(n):
+        ths.append(Process(target=_make_df_part, args=(divided_list[i], q)))
+    for i in range(n):
+        ths[i].start()
+    df_list = []
+    for i in range(n):
+        df_list.append(q.get())
+    # 부분데이터프레임들을 하나로 합침
+    final_df = pd.concat(df_list, ignore_index=True)
+    for i in range(n):
+        ths[i].join()
+    print(f'Total spent time : {round(time.time() - start_time, 2)} sec.')
+    logger.debug(final_df)
+    return final_df
